#include "NSGAII_Steady_MIES.h"

//------------------------------------------------------------------------------
// Class NSGAII_Steady_MIES
//-------------------------------------------------
// Protected Members

/* Create lambda offspring by mutation of a single (recombined) parent */
void NSGAII_Steady_MIES::recombineMutate()
{
#ifdef DEBUG
  cout << "NSGAII_Steady_MIES::recombineMutate()" << endl;
#endif

  MixedIntegerES::recombineMutateSingleParent();
}

/* Select mu Individuals from O and optionally P to form new population P */
void NSGAII_Steady_MIES::select()
{
#ifdef DEBUG
  cout << "NSGAII_Steady_MIES::select()" << endl;
#endif

  /* Select "best" offspring individual, is placed at position 0 in offspring vector O */
  preSelect();

  NSGAII_MIES::select();
}

/* Select the "best" offspring Individual from the offspring pool of size lambda, using comparison score */
void NSGAII_Steady_MIES::preSelect()
{
#ifdef DEBUG
  cout << "NSGAII_Steady_MIES::preSelect()" << endl;
#endif

  /* Build pool for comparing offspring Individuals */
  vector<Individual*> Q;
  for (unsigned i = (excludeParent) ? 1 : 0; i < mu; i++)
    Q.push_back(P[i]);
  for (unsigned i = 0; i < lambda; i++)
    Q.push_back(O[i]);	  

  /* Calculate scores for comparing the offspring Individuals */
  map<int, double> score; 
  prepComparisonScore(Q); // Prepare for calculating comparison scores (if necessary)
  for (unsigned i = 0; i < lambda; i++)
    score[i] = calcComparisonScore(i);

  vector<int> indices; // Indices of the lambda offspring Individuals in O
  for (unsigned i = 0; i < lambda; i++)
    indices.push_back(i);	  

  /* Sort indices based on comparsison score, in descending order */
  quickSort(indices, 0, lambda-1, NULL, -1, &score);

  /* Move best offspring individual to position 0 in offspring vector O */ 
  if (indices[0] != 0)
  {
    delete O[0];
    O[0] = O[indices[0]];
    O[indices[0]] = NULL;
  }

  /* Cleanup rest of vector */
  for (unsigned i = 1; i < lambda; i++)
  {
    delete O[i];
    O[i] = NULL;
  }
}


