#ifndef NSGAII_STEADY_MIES
#define NSGAII_STEADY_MIES

#include "NSGAII_MIES.h"
/** \brief  Steady state (mu + (1, lambda))-MIES version of NSGA-II, abstract class 
 *          (also see NSGAII_MIES.h) */
class NSGAII_Steady_MIES:virtual public NSGAII_MIES 
{
  public:
    NSGAII_Steady_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
		        unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
		         vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
		           vector<int>& lBound_d_, vector<int>& uBound_d_, double prob_min_, double prob_max_,
		             bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
		               bool selfAdaptation_, vector<int>& direction_, 
		                 unsigned evaluations_, vector<double>& optimalF_, 
			           double initialSigma_r_, double initialSigma_z_, double initialProb_,
			             ifstream* initialPopFile_, int randomSeed_, bool feedback_, 
			               unsigned timeOutValue_,
		    	                 unsigned selectDimension_, vector<unsigned>& selectFunction_,
					   bool excludeParent_) 
      :NSGAII_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		    mu_, rho_, kappa_, lambda_,  
		      lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		        lBound_d_, uBound_d_, prob_min_, prob_max_,
		          disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
		            selfAdaptation_, direction_, 
		              evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
		                initialPopFile_, randomSeed_, feedback_, timeOutValue_,
		                  selectDimension_, selectFunction_) 
	                          {
#ifdef DEBUG
                                    cout << "NSGAII_Steady_MIES::NSGAII_Steady_MIES()" << endl;
#endif	 
				    // Only allow to exclude single parent from comparison score calculation (see calcComparisonScore())
				    // if the single parent is not the result of recombination 
				    // (i.e., only when rho=1, see MixedIntegerES::recombineMutateSingleParent()) 
			            if (rho > 1)
  				      excludeParent = excludeParent_;
				    else
				      excludeParent = false;	    
				  }
    virtual ~NSGAII_Steady_MIES() {}

  protected:
    /** \brief  Determines whether parent individual is excluded from comparison score calculation in preSelect() */
    bool excludeParent; 

    /** \brief  Create lambda offspring by mutation of a single parent */
    virtual void recombineMutate();

    /** \brief  Select mu Individuals from O and optionally P to form new population P */
    void select(); 

    /** \brief  Select the "best" offspring Individual from the offspring pool of size lambda, using comparison score */
    void preSelect();

    /** \brief  Prepare for calculating comparison scores (if necessary) */
    virtual void prepComparisonScore(vector<Individual*>& Q) = 0;

    /** \brief  Calculate comparison score for pre-selecting "best" offspring Individual */ 
    virtual double calcComparisonScore(unsigned offspringIndex) = 0;
};

#endif
