#ifndef SMS3D_
#define SMS3D_

#include "NSGA_II.h"

extern "C" 
{
#include "hv.h"
}

/** \brief  Generational version of 3D SMS-EMOA, optionally using dp selection
 *          (see Beume, 2007 - SMS-EMOA) */
class SMSEMOA_3D:virtual public NSGA_II
{
  public:
    SMSEMOA_3D(bool dpSelection_, bool steadyState_, unsigned n_f__ = 3);
    virtual ~SMSEMOA_3D() {}

  protected: 
    bool steadyState;
    double reference[3];
    map<int, vector<double> > normF;

    /** \brief  Sort Individuals within same front: 3D hypervolume sorting (see Complexity of Hypervolume by Beume, 2007) */
    void frontSort(vector<int>& currFront, vector<Individual*>& Q, vector<int>& direction); 

    /** \brief  Normalize F values for Individuals in currFront to [0,1]->min, using Nadir and Ideal point */
    void normalize(vector<int>& currFront, int frontSize, vector<Individual*>& Q, vector<int>& direction); 
};

#endif

