#ifndef SMS_STEADY3D_MIES
#define SMS_STEADY3D_MIES

#include "NSGAII_Steady_MIES.h"
#include "SMSEMOA_3D.h"

/** \brief  Steady state (mu + (1, lambda))-MIES version of 3D SMS-EMOA, optionally using dp selection; abstract class
 *          (see Beume, 2007 - SMS-EMOA; see NSGAII_MIES.h) */
class SMSEMOA_Steady3D_MIES:public NSGAII_Steady_MIES, public SMSEMOA_3D 
{                                                                         
  public:                                                                
    SMSEMOA_Steady3D_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
		           unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
		             vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
		               vector<int>& lBound_d_, vector<int>& uBound_d_, double prob_min_, double prob_max_,
		                 bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
		                   bool selfAdaptation_, vector<int>& direction_, 
		                     unsigned evaluations_, vector<double>& optimalF_, 
			               double initialSigma_r_, double initialSigma_z_, double initialProb_,
			                 ifstream* initialPopFile_, int randomSeed_, bool feedback_, 
			                   unsigned timeOutValue_,
		    	                     unsigned selectDimension_, vector<unsigned>& selectFunction_,
					       bool excludeParent_) 
      :NSGAII_Steady_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		           mu_, rho_, kappa_, lambda_,  
		             lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		               lBound_d_, uBound_d_, prob_min_, prob_max_,
			         disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			           selfAdaptation_, direction_, 
			             evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
			               initialPopFile_, randomSeed_, feedback_, timeOutValue_,
			                 selectDimension_, selectFunction_,
					   excludeParent_),
       NSGAII_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		    mu_, rho_, kappa_, lambda_,  
		      lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		        lBound_d_, uBound_d_, prob_min_, prob_max_,
		          disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
		            selfAdaptation_, direction_, 
		              evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
		                initialPopFile_, randomSeed_, feedback_, timeOutValue_,
		                  selectDimension_, selectFunction_),
#ifdef DP
       SMSEMOA_3D(true, true, n_f_) 
#else
       SMSEMOA_3D(false, true, n_f_) 
#endif
       {
#ifdef DEBUG
         cout << "SMSEMOA_Steady3D_MIES::SMSEMOA_Steady3D_MIES()" << endl;
#endif	 
	 numIndividuals = (excludeParent) ? mu : mu + 1;
         dataHv = new double[numIndividuals*3];
       }

    ~SMSEMOA_Steady3D_MIES() { delete [] dataHv; }

  protected:
    unsigned numIndividuals;
    double* dataHv; 
    
    /** \brief  Normalize Individuals in Q and prepare dataHv */
    void prepComparisonScore(vector<Individual*>& Q); 

    /** \brief  Calculates 3D HV contr. approximations for pre-selecting "best" offspring Individual */ 
    double calcComparisonScore(unsigned offspringIndex);
};

#endif
