#ifndef GLOBAL
#define GLOBAL

#include <cstdlib>
#include <cstring>
#include <iostream>
#include <vector>
#include <cmath>
#include <ctime>
#include <climits>
#include <cfloat>
#include <algorithm>
#include <sstream>
#include <fstream>

#include <boost/random/uniform_int.hpp>
#include <boost/random/uniform_real.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/math/distributions/normal.hpp>
#include <boost/lexical_cast.hpp>

#define HEADER_CONSTANTS "[CONSTANT SIMULATOR PARAMETERS]"
#define HEADER_NAMES     "[VARIABLE SIMULATOR PARAMETER NAMES]"
#define HEADER_MIES      "[MIES PARAMETERS]"
#define HEADER_SIMULATOR "[SIMULATOR]"
#define HEADER_PARALLEL  "[PARALELLIZATION]"
#define LEN              1024 // Number of characters read per line from input file
#define SPACE            "#"
#define REPEAT           '%'
#define COMMENT          ';'
#define DELIMS           " ;\t"

#define DEBUG_FROM_GEN   4000 // In case of slow convergence, show debug info

using namespace std;

typedef boost::mt19937 base_generator_type;

extern base_generator_type randGenerator;
extern boost::uniform_real<> uniDistReal;
extern boost::variate_generator<base_generator_type&, boost::uniform_real<> > uniReal;
extern boost::uniform_int<> uniDistInt;
extern boost::variate_generator<base_generator_type&, boost::uniform_int<> > uniInt;
extern boost::normal_distribution<> normDist;
extern boost::variate_generator<base_generator_type&, boost::normal_distribution<> > normReal;
extern boost::math::normal_distribution<> normDistMath;

void randPerm(vector<int>& A, int n); // Generate random permutation of 1,...,n in vector A 
string itos(int number);
void skipLinesUntil(ifstream& infile, string text);
char* getValue(ifstream& infile);
string getStringValue(ifstream& infile);
void getBound(ifstream& infile, vector<double>* bound_r, vector<int>* bound_d_z, int n);
void findAndReplace(string& text, const char* find, const char* replace);

#endif
