#include "main.h"

int main(int argc, char* argv[])
{ 
  if (argc < 2) 
  { 
    cerr << "No input file supplied! Exiting." << endl;	  
    return 1;
  }

  char line[LEN];
  ifstream infile(argv[1]);

  /* Read constant simulator parameters from input file */
  vector<string> constSimParams;
  skipLinesUntil(infile, HEADER_CONSTANTS);

  infile.getline(line, LEN);
  while (line[0] != '\0')
  {
    constSimParams.push_back(strtok(line, " ;"));
    findAndReplace(constSimParams.back(), SPACE, " ");
    infile.getline(line, LEN);
  }

  /* Read variable simulator parameter names from input file */	
  vector<string> varSimParamNames;
  skipLinesUntil(infile, HEADER_NAMES);

  infile.getline(line, LEN);
  while (line[0] != '\0')
  {
    varSimParamNames.push_back(strtok(line, " ;"));
    infile.getline(line, LEN);
  }

  /* Read MIES parameters from input file */
  skipLinesUntil(infile, HEADER_MIES);
  
  unsigned n_r       = atoi(getValue(infile)),
           n_z       = atoi(getValue(infile)),
           n_d       = atoi(getValue(infile)),
           n_sigma_r = atoi(getValue(infile)),
           n_sigma_z = atoi(getValue(infile)),
           n_prob    = atoi(getValue(infile)),
           n_f       = atoi(getValue(infile)),
	   mu        = atoi(getValue(infile)),
	   rho       = atoi(getValue(infile)),
           kappa     = atoi(getValue(infile)),
           lambda    = atoi(getValue(infile));

  vector<double> lBound_r, uBound_r; 
  vector<int> lBound_z, uBound_z, lBound_d, uBound_d; 

  getBound(infile, &lBound_r, NULL, n_r);
  getBound(infile, &uBound_r, NULL, n_r);

  getBound(infile, NULL, &lBound_z, n_z);
  getBound(infile, NULL, &uBound_z, n_z);

  getBound(infile, NULL, &lBound_d, n_d);
  getBound(infile, NULL, &uBound_d, n_d);

  double prob_min = atof(getValue(infile)),
	 prob_max = atof(getValue(infile));

  bool disRec_r       = (atoi(getValue(infile)) == 0) ? false : true,
       disRec_z       = (atoi(getValue(infile)) == 0) ? false : true, 
       disRec_sigma_r = (atoi(getValue(infile)) == 0) ? false : true,
       disRec_sigma_z = (atoi(getValue(infile)) == 0) ? false : true, 
       disRec_prob    = (atoi(getValue(infile)) == 0) ? false : true,
       selfAdaptation = (atoi(getValue(infile)) == 0) ? false : true; 

  vector<int> direction; 
  for (unsigned i = 0; i < n_f; i++)
    direction.push_back(atoi(getValue(infile)));

  unsigned selectDimension = atoi(getValue(infile));

  vector<unsigned> selectFunction;
  for (unsigned i = 0; i < selectDimension; i++)
    selectFunction.push_back(atoi(getValue(infile)));

  unsigned evaluations = atoi(getValue(infile));

  vector<double> optimalF;
  for (unsigned i = 0; i < n_f; i++)
    optimalF.push_back(atof(getValue(infile)));

  double initialSigma_r = atof(getValue(infile)),
	 initialSigma_z = atof(getValue(infile)),
         initialProb    = atof(getValue(infile));

  string initialPopFileName = getStringValue(infile);

  int randomSeed = atoi(getValue(infile));

  bool feedback = (atoi(getValue(infile)) == 1) ? true : false;

  string ES = getStringValue(infile);

  string MOproblem;
  if (ES == "TestFunctions_MIES")
    MOproblem = getStringValue(infile);

  bool excludeParent;
#ifdef STEADY
  excludeParent = (atoi(getValue(infile)) == 0) ? false : true;
#endif

  /* Read simulator file names from input file */
  skipLinesUntil(infile, HEADER_SIMULATOR);
  string simulator         = getStringValue(infile),
	 path              = getStringValue(infile),
	 simInputFileName  = getStringValue(infile),
	 simOutputFileName = getStringValue(infile);

  unsigned timeOutValue = 0;
#ifdef PARALLEL
  /* Read time out value from input file */
  skipLinesUntil(infile, "server");
  timeOutValue = atoi(getValue(infile));
#endif
  infile.close();

  findAndReplace(initialPopFileName, SPACE, " ");
  findAndReplace(MOproblem, SPACE, " ");
  findAndReplace(simulator, SPACE, " ");
  findAndReplace(path, SPACE, " ");
  findAndReplace(simInputFileName, SPACE, " ");
  findAndReplace(simOutputFileName, SPACE, " ");

  ifstream* initialPopFile = (initialPopFileName == " ") ? NULL : new ifstream(initialPopFileName.c_str());

  simulator = "\"\"\"" + simulator + "\" \"" + path + "\"\"\"";  
 
#ifndef PARALLEL
  if (ES == "TestFunctions_MIES")
  {
    TestFunctions_MIES testMies(n_r, n_z, n_d, n_sigma_r, n_sigma_z, n_prob, n_f, 
		                 mu, rho, kappa, lambda,  
		                   lBound_r, uBound_r, lBound_z, uBound_z, 
		                     lBound_d, uBound_d, prob_min,  prob_max,
			               disRec_r, disRec_z, disRec_sigma_r, disRec_sigma_z, disRec_prob,
			                 selfAdaptation, direction, 
			                   evaluations, optimalF, initialSigma_r, initialSigma_z, initialProb, initialPopFile, 
					     randomSeed, feedback, timeOutValue,
				               selectDimension, selectFunction,
					         MOproblem, excludeParent);
    testMies.run();
  }
#endif
  
#ifdef MASPROJ
  if (ES == "SingleAisle_MIES")
  {	  
# ifdef PARALLEL
    SingleAisle_MIES_P saMies(n_r, n_z, n_d, n_sigma_r, n_sigma_z, n_prob, n_f, 
      		               mu, rho, kappa, lambda,  
		                 lBound_r, uBound_r, lBound_z, uBound_z, 
		                   lBound_d, uBound_d, prob_min, prob_max,
		                     disRec_r, disRec_z, disRec_sigma_r, disRec_sigma_z, disRec_prob,
			               selfAdaptation, direction, 
			                 evaluations, optimalF, initialSigma_r, initialSigma_z, initialProb, initialPopFile, 
					   randomSeed, feedback, timeOutValue,
			       	             simulator, simInputFileName, simOutputFileName,
				               constSimParams, varSimParamNames,
					         selectDimension, selectFunction);
# else
    SingleAisle_MIES saMies(n_r, n_z, n_d, n_sigma_r, n_sigma_z, n_prob, n_f, 
      		             mu, rho, kappa, lambda,  
		               lBound_r, uBound_r, lBound_z, uBound_z, 
		                 lBound_d, uBound_d, prob_min, prob_max,
		                   disRec_r, disRec_z, disRec_sigma_r, disRec_sigma_z, disRec_prob,
			             selfAdaptation, direction, 
			               evaluations, optimalF, initialSigma_r, initialSigma_z, initialProb, initialPopFile, 
				         randomSeed, feedback, timeOutValue,
			       	           simulator, simInputFileName, simOutputFileName,
				             constSimParams, varSimParamNames,
					       selectDimension, selectFunction);
# endif
    saMies.run();
  }

  else if (ES == "MultiAisle_MIES")
  {	  
# ifdef PARALLEL
    MultiAisle_MIES_P maMies(n_r, n_z, n_d, n_sigma_r, n_sigma_z, n_prob, n_f, 
      		               mu, rho, kappa, lambda,  
		                 lBound_r, uBound_r, lBound_z, uBound_z, 
		                   lBound_d, uBound_d, prob_min, prob_max,
		                     disRec_r, disRec_z, disRec_sigma_r, disRec_sigma_z, disRec_prob,
			               selfAdaptation, direction, 
			                 evaluations, optimalF, initialSigma_r, initialSigma_z, initialProb, initialPopFile, 
					   randomSeed, feedback, timeOutValue,
			       	             simulator, simInputFileName, simOutputFileName,
				               constSimParams, varSimParamNames,
					         selectDimension, selectFunction);
# else
    MultiAisle_MIES maMies(n_r, n_z, n_d, n_sigma_r, n_sigma_z, n_prob, n_f, 
      		             mu, rho, kappa, lambda,  
		               lBound_r, uBound_r, lBound_z, uBound_z, 
		                 lBound_d, uBound_d, prob_min, prob_max,
		                   disRec_r, disRec_z, disRec_sigma_r, disRec_sigma_z, disRec_prob,
			             selfAdaptation, direction, 
			               evaluations, optimalF, initialSigma_r, initialSigma_z, initialProb, initialPopFile, 
				         randomSeed, feedback, timeOutValue,
			       	           simulator, simInputFileName, simOutputFileName,
				             constSimParams, varSimParamNames,
					       selectDimension, selectFunction);
# endif
    maMies.run();
  }
#endif

  return 0;
}

