#include <iostream>
#include <vector>
#include <stdio.h>

extern "C" 
{
#include "global.h"
#include "rand.h"
}
#include "../mies/SerialES/SMSEMOA_3D.h"

using namespace std;

#ifdef DP
SMSEMOA_3D sms3d(true, true);  // 3D SMS-EMOA using dominating points (dp) selection, see Beume, 2007 - SMS-EMOA 
#else
SMSEMOA_3D sms3d(false, true); // 3D SMS-EMOA 
#endif

int rndExcl(int min, int max, int exclude)
{
  int rand;	
  do
  {
    rand = rnd(min, max);
  }
  while (rand == exclude);

  return rand;
}

void convert(population* pop, individual* ind, vector<Individual*>& P)
{
  int size = (pop != NULL) ? popsize : 1;

  for (int i = 0; i < size; i++)
  {
    P.push_back(new Individual());
    
    for (int j = 0; j < nreal; j++)
      P[i]->R.push_back((pop != NULL) ? pop->ind[i].xreal[j] : ind->xreal[j]);

    for (int j = 0; j < nbin; j++)
      P[i]->Z.push_back((pop != NULL) ? (int) pop->ind[i].xbin[j] : (int) ind->xbin[j]); // Discard real part of xbin vars

    for (int j = 0; j < nnom; j++)
      P[i]->D.push_back((pop != NULL) ? pop->ind[i].xnom[j] : ind->xnom[j]);

    for (int j = 0; j < 3; j++)
      P[i]->F.push_back((pop != NULL) ? pop->ind[i].obj[j] : ind->obj[j]);
  }
}

void revert(population* pop, vector<Individual*>& P)
{
  for (int i = 0; i < popsize; i++)
  {
    for (int j = 0; j < nreal; j++)
      pop->ind[i].xreal[j] = P[i]->R[j];

    for (int j = 0; j < nbin; j++)
      pop->ind[i].xbin[j] = P[i]->Z[j];

    for (int j = 0; j < nnom; j++)
      pop->ind[i].xnom[j] = P[i]->D[j];

    for (int j = 0; j < 3; j++)
      pop->ind[i].obj[j] = P[i]->F[j];

    delete P[i];

    decode_ind(&pop->ind[i]);
  }
}

EXTERN_C void setNadirIdeal(double* nadir, double* ideal)
{
  vector<double> nadir_, ideal_;
  for (int i = 0; i < 3; i++)
  {
    nadir_.push_back(nadir[i]);
    ideal_.push_back(ideal[i]);
  }  

  sms3d.setNadirIdeal(nadir_, ideal_);
}

EXTERN_C void generate(population* pop, individual* ind)
{
  int rand1 = rnd(0, popsize-1);
  int rand2 = rndExcl(0, popsize-1, rand1);
  individual* parent1 = tournament(&pop->ind[rand1], &pop->ind[rand2]),
	    * parent2;

  do
  {
    rand1 = rnd(0, popsize-1);
    rand2 = rndExcl(0, popsize-1, rand1);
    parent2 = tournament(&pop->ind[rand1], &pop->ind[rand2]);
  }
  while (parent2 == parent1);

  crossover(parent1, parent2, ind, NULL);
  mutation_ind(ind);
  decode_ind(ind);
  evaluate_ind(ind);
}

EXTERN_C void reduce(population* pop, individual* ind)
{
  vector<Individual*> P, O;
  convert(pop, NULL, P);
  convert(NULL, ind, O);	  
  
  unsigned mu     = popsize,
	   kappa  = INT_MAX, // i.e., (mu+lambda)-selection
	   lambda = 1; 

  vector<int> direction;
  vector<double> bestF;
  for (int i = 0; i < 3; i++) 
  {
    direction.push_back(obj_direction[i]);

    if (direction[i] == 1)
      bestF.push_back(0);
    else
      bestF.push_back(LONG_MAX);
  }

  sms3d.select(P, O, mu, kappa, lambda, direction, bestF);

  // TODO: process bestF

  revert(pop, P);
}

