#!/bin/csh -f

if ($# < 3) then
  echo "Usage: $0 WDNproblem *attData.txt RunID [b|m|w]"
  exit
endif

set plotDir     = "plot"
set ghostScript = "gs"
set gnuplot     = "gnuplot"
set plotAtt     =  "$plotDir/plot-att"

if ($1 == "T") then
  set WDNproblem = "TwoLoop"
else if ($1 == "H") then
  set WDNproblem = "Hanoi"
else if ($1 == "N") then
  set WDNproblem = "NewYorkCity"
endif  

if ($# == 3) then
  set best   = 1
  set median = 1 
  set worst  = 1
else
 if ($4 == "b") then
   set best   = 1
   set median = 0 
   set worst  = 0
 else if ($4 == "m") then
   set best   = 0
   set median = 1 
   set worst  = 0
 else if ($4 == "w") then
   set best   = 0
   set median = 0 
   set worst  = 1
 endif
endif

if (! -e $plotAtt) then
  g++ $plotDir/plot-attainments2.cc -o $plotDir/plot-att -lm -O3
  sleep 1
endif

if ($best == 1) then
  $plotAtt $2 -b $plotDir/bound$1.txt -k 3 -mm -1 1 1 -r 400 -a 1 > $plotDir/best.txt
endif

if ($median == 1) then
  $plotAtt $2 -b $plotDir/bound$1.txt -k 3 -mm -1 1 1 -r 400 -a 15 > $plotDir/median.txt
endif

if ($worst == 1) then 
  $plotAtt $2 -b $plotDir/bound$1.txt -k 3 -mm -1 1 1 -r 400 -a 30 > $plotDir/worst.txt
endif

$gnuplot $plotDir/attain_$WDNproblem.plt

if ($best == 1) then
  "$ghostScript" -dSAFER -dBATCH -dNOPAUSE -r200 -sDEVICE=png16m -dGraphicsAlphaBits=4 \
      -sOutputFile=$plotDir/$3_b.png $plotDir/best.ps
endif

if ($median == 1) then
  "$ghostScript" -dSAFER -dBATCH -dNOPAUSE -r200 -sDEVICE=png16m -dGraphicsAlphaBits=4 \
      -sOutputFile=$plotDir/$3_m.png $plotDir/median.ps
endif

if ($worst == 1) then 
  "$ghostScript" -dSAFER -dBATCH -dNOPAUSE -r200 -sDEVICE=png16m -dGraphicsAlphaBits=4 \
      -sOutputFile=$plotDir/$3_w.png $plotDir/worst.ps
endif

rm -f best.* median.* worst.* > /dev/null
